@@ -29,22 +29,14 @@ class DialectPreferencesWindow(Adw.PreferencesWindow):
     translate_accel = Gtk.Template.Child()
     src_auto = Gtk.Template.Child()
     backend = Gtk.Template.Child()
-    backend_instance = Gtk.Template.Child()
-    backend_instance_row = Gtk.Template.Child()
-    backend_instance_stack = Gtk.Template.Child()
-    backend_instance_label = Gtk.Template.Child()
-    backend_instance_edit = Gtk.Template.Child()
-    backend_instance_save = Gtk.Template.Child()
-    backend_instance_reset = Gtk.Template.Child()
-    backend_instance_edit_box = Gtk.Template.Child()
-    api_key = Gtk.Template.Child()
-    api_key_row = Gtk.Template.Child()
+    instance_entry = Gtk.Template.Child()
+    instance_stack = Gtk.Template.Child()
+    instance_reset = Gtk.Template.Child()
+    instance_spinner = Gtk.Template.Child()
+    api_key_entry = Gtk.Template.Child()
     api_key_stack = Gtk.Template.Child()
-    api_key_label = Gtk.Template.Child()
-    api_key_edit = Gtk.Template.Child()
-    api_key_save = Gtk.Template.Child()
     api_key_reset = Gtk.Template.Child()
-    api_key_edit_box = Gtk.Template.Child()
+    api_key_spinner = Gtk.Template.Child()
     tts = Gtk.Template.Child()
     tts_row = Gtk.Template.Child()
     search_provider = Gtk.Template.Child()
@@ -98,46 +90,20 @@ class DialectPreferencesWindow(Adw.PreferencesWindow):
         # Toggle TTS
         self.tts.connect('notify::active', self._toggle_tts)
 
-        # Change translator instance
+        # Connect to settings changes
         Settings.get().connect('changed', self._on_settings_changed)
-        self.backend_instance_edit.connect('clicked', self._on_edit_backend_instance)
-        self.backend_instance_save.connect('clicked', self._on_save_backend_instance)
-        self.backend_instance_reset.connect('clicked', self._on_reset_backend_instance)
 
-        self.api_key_edit.connect('clicked', self._on_edit_api_key)
-        self.api_key_save.connect('clicked', self._on_save_api_key)
+        # Instance
+        self.instance_entry.connect('apply', self._on_instance_apply)
+        self.instance_entry.connect('changed', self._on_instance_changed)
+        self.instance_reset.connect('clicked', self._on_reset_instance)
+
+        # API Key
+        self.api_key_entry.connect('apply', self._on_api_key_apply)
         self.api_key_reset.connect('clicked', self._on_reset_api_key)
 
         self.__check_instance_or_api_key_support()
 
-        self.instance_save_image = Gtk.Image.new_from_icon_name('emblem-ok-symbolic')
-        self.backend_instance_save.set_child(self.instance_save_image)
-        self.instance_save_spinner = Gtk.Spinner()
-
-        self.api_key_save_image = Gtk.Image.new_from_icon_name('emblem-ok-symbolic')
-        self.api_key_save.set_child(self.api_key_save_image)
-        self.api_key_save_spinner = Gtk.Spinner()
-
-        self.error_popover = Gtk.Popover(
-            pointing_to=self.backend_instance.get_allocation(),
-            can_focus=False,
-        )
-        self.error_label = Gtk.Label(label='Not a valid instance')
-        error_icon = Gtk.Image.new_from_icon_name('dialog-error-symbolic')
-        error_box = Gtk.Box(
-            orientation=Gtk.Orientation.HORIZONTAL,
-            margin_start=8,
-            margin_end=8,
-            margin_top=8,
-            margin_bottom=8,
-            spacing=8
-        )
-        error_box.prepend(error_icon)
-        error_box.prepend(self.error_label)
-        self.error_popover.set_child(error_box)
-        self.error_popover.set_position(Gtk.PositionType.BOTTOM)
-        self.error_popover.hide()
-
         # Search Provider
         if os.getenv('XDG_CURRENT_DESKTOP') != 'GNOME':
             self.search_provider.hide()
@@ -178,6 +144,7 @@ class DialectPreferencesWindow(Adw.PreferencesWindow):
             ).start()
 
     def _switch_backends(self, row, _value):
+        self.parent.save_settings()
         backend = self.backend_model[row.get_selected()].name
         Settings.get().active_translator = backend
         self.__check_instance_or_api_key_support()
@@ -185,22 +152,24 @@ class DialectPreferencesWindow(Adw.PreferencesWindow):
 
     def _on_backend_loading(self, window, _value):
         self.backend.set_sensitive(not window.get_property('backend-loading'))
-        self.backend_instance_row.set_sensitive(not window.get_property('backend-loading'))
-        self.api_key_row.set_sensitive(not window.get_property('backend-loading'))
+        self.instance_entry.set_sensitive(not window.get_property('backend-loading'))
+        self.api_key_entry.set_sensitive(not window.get_property('backend-loading'))
 
-        # Show or hide api key entry
-        if not window.get_property('backend-loading') and window.translator:
-            if window.translator.supported_features['api-key-supported']:
-                self.api_key_row.set_visible(True)
-                self.api_key_label.set_label(Settings.get().api_key or 'None')
-            else:
-                self.api_key_row.set_visible(False)
+        if not window.get_property('backend-loading'):
+            self.instance_stack.set_visible_child_name('reset')
+            self.instance_spinner.stop()
+            self.api_key_stack.set_visible_child_name('reset')
+            self.api_key_spinner.stop()
 
-    def _on_edit_backend_instance(self, _button):
-        self.backend_instance_stack.set_visible_child_name('edit')
-        self.backend_instance.set_text(Settings.get().instance_url)
+            # Show or hide api key entry
+            if window.translator:
+                if window.translator.supported_features['api-key-supported']:
+                    self.api_key_entry.set_visible(True)
+                    self.api_key_entry.set_text(Settings.get().api_key)
+                else:
+                    self.api_key_entry.set_visible(False)
 
-    def _on_save_backend_instance(self, _button):
+    def _on_instance_apply(self, _row):
         def on_validation_response(session, result):
             valid = False
             backend = Settings.get().active_translator
@@ -212,27 +181,25 @@ class DialectPreferencesWindow(Adw.PreferencesWindow):
 
             if valid:
                 Settings.get().instance_url = self.new_instance_url
-                self.backend_instance.get_style_context().remove_class('error')
-                self.backend_instance_stack.set_visible_child_name('view')
-                # self.error_popover.popdown()
+                self.instance_entry.remove_css_class('error')
+                self.instance_entry.set_text(Settings.get().instance_url)
             else:
-                self.backend_instance.get_style_context().add_class('error')
+                self.instance_entry.add_css_class('error')
                 error_text = _('Not a valid {backend} instance')
                 error_text = error_text.format(backend=TRANSLATORS[backend].prettyname)
-                self.error_label.set_label(error_text)
-                # self.error_popover.popup()
-                self.api_key_row.set_visible(False)
-                self.api_key_label.set_label('None')
+                toast = Adw.Toast.new(error_text)
+                self.add_toast(toast)
+                self.api_key_entry.set_visible(False)
+                self.api_key_entry.set_text('')
 
             self.backend.set_sensitive(True)
-            self.backend_instance_row.set_sensitive(True)
-            self.api_key_row.set_sensitive(True)
-            self.backend_instance_save.set_child(self.instance_save_image)
-            self.backend_instance_label.set_label(Settings.get().instance_url)
-            self.instance_save_spinner.stop()
+            self.instance_entry.set_sensitive(True)
+            self.api_key_entry.set_sensitive(True)
+            self.instance_stack.set_visible_child_name('reset')
+            self.instance_spinner.stop()
 
         old_value = Settings.get().instance_url
-        new_value = self.backend_instance.get_text()
+        new_value = self.instance_entry.get_text()
 
         url = re.compile(r'https?://(www\.)?')
         self.new_instance_url = url.sub('', new_value).strip().strip('/')
@@ -241,10 +208,10 @@ class DialectPreferencesWindow(Adw.PreferencesWindow):
         if self.new_instance_url != old_value:
             # Progress feedback
             self.backend.set_sensitive(False)
-            self.backend_instance_row.set_sensitive(False)
-            self.api_key_row.set_sensitive(False)
-            self.backend_instance_save.set_child(self.instance_save_spinner)
-            self.instance_save_spinner.start()
+            self.instance_entry.set_sensitive(False)
+            self.api_key_entry.set_sensitive(False)
+            self.instance_stack.set_visible_child_name('spinner')
+            self.instance_spinner.start()
 
             backend = Settings.get().active_translator
             validation_url = TRANSLATORS[backend].format_url(
@@ -255,21 +222,25 @@ class DialectPreferencesWindow(Adw.PreferencesWindow):
 
             Session.get().send_and_read_async(validation_message, 0, None, on_validation_response)
         else:
-            self.backend_instance_stack.set_visible_child_name('view')
+            self.instance_entry.remove_css_class('error')
 
-    def _on_reset_backend_instance(self, _button):
-        Settings.get().reset_instance_url()
-        Settings.get().reset_api_key()
-        self.backend_instance_label.set_label(Settings.get().instance_url)
-        self.backend_instance_stack.set_visible_child_name('view')
-        self.backend_instance.get_style_context().remove_class('error')
-        self.error_popover.popdown()
+    def _on_instance_changed(self, _entry):
+        if self.instance_entry.props.text == Settings.get().instance_url:
+            self.instance_entry.props.show_apply_button = False
+        elif not self.instance_entry.props.show_apply_button:
+            self.instance_entry.props.show_apply_button = True
 
-    def _on_edit_api_key(self, _button):
-        self.api_key_stack.set_visible_child_name('edit')
-        self.api_key.set_text(Settings.get().api_key)
+    def _on_reset_instance(self, _button):
+        backend = Settings.get().active_translator
+        if Settings.get().instance_url != TRANSLATORS[backend].instance_url:
+            Settings.get().reset_instance_url()
+            Settings.get().reset_api_key()
+            self.instance_stack.set_visible_child_name('spinner')
+            self.instance_spinner.start()
+        self.instance_entry.remove_css_class('error')
+        self.instance_entry.set_text(Settings.get().instance_url)
 
-    def _on_save_api_key(self, _button):
+    def _on_api_key_apply(self, _row):
         def on_response(session, result):
             valid = False
             try:
@@ -281,31 +252,32 @@ class DialectPreferencesWindow(Adw.PreferencesWindow):
 
             if valid:
                 Settings.get().api_key = self.new_api_key
-                self.api_key.get_style_context().remove_class('error')
-                self.api_key_stack.set_visible_child_name('view')
+                self.api_key_entry.remove_css_class('error')
+                self.api_key_entry.set_text(Settings.get().instance_url)
             else:
-                self.api_key.get_style_context().add_class('error')
+                self.api_key_entry.add_css_class('error')
                 error_text = _('Not a valid {backend} API key')
                 error_text = error_text.format(backend=TRANSLATORS[backend].prettyname)
-                self.error_label.set_label(error_text)
+                toast = Adw.Toast.new(error_text)
+                self.add_toast(toast)
+                self.api_key_entry.grab_focus()
 
             self.backend.set_sensitive(True)
-            self.backend_instance_row.set_sensitive(True)
-            self.api_key_row.set_sensitive(True)
-            self.api_key_save.set_child(self.api_key_save_image)
-            self.api_key_label.set_label(Settings.get().api_key or 'None')
-            self.instance_save_spinner.stop()
+            self.instance_entry.set_sensitive(True)
+            self.api_key_entry.set_sensitive(True)
+            self.api_key_stack.set_visible_child_name('reset')
+            self.api_key_spinner.stop()
 
         old_value = Settings.get().api_key
-        self.new_api_key = self.api_key.get_text()
+        self.new_api_key = self.api_key_entry.get_text()
 
         if self.new_api_key != old_value:
             # Progress feedback
             self.backend.set_sensitive(False)
-            self.backend_instance_row.set_sensitive(False)
-            self.api_key_row.set_sensitive(False)
-            self.api_key_save.set_child(self.api_key_save_spinner)
-            self.api_key_save_spinner.start()
+            self.instance_entry.set_sensitive(False)
+            self.api_key_entry.set_sensitive(False)
+            self.api_key_stack.set_visible_child_name('spinner')
+            self.api_key_spinner.start()
 
             backend = Settings.get().active_translator
             validation_url = TRANSLATORS[backend].format_url(
@@ -317,28 +289,30 @@ class DialectPreferencesWindow(Adw.PreferencesWindow):
 
             Session.get().send_and_read_async(message, 0, None, on_response)
         else:
-            self.api_key_stack.set_visible_child_name('view')
+            self.api_key_entry.remove_css_class('error')
 
     def _on_reset_api_key(self, _button):
-        Settings.get().reset_api_key()
-        self.api_key_label.set_label(Settings.get().api_key or 'None')
-        self.api_key_stack.set_visible_child_name('view')
-        self.api_key.get_style_context().remove_class('error')
-        self.error_popover.popdown()
+        backend = Settings.get().active_translator
+        if Settings.get().api_key != TRANSLATORS[backend].api_key:
+            Settings.get().reset_api_key()
+            self.api_key_stack.set_visible_child_name('spinner')
+            self.api_key_spinner.start()
+        self.api_key_entry.remove_css_class('error')
+        self.api_key_entry.set_text(Settings.get().api_key)
 
     def __check_instance_or_api_key_support(self):
         backend = Settings.get().active_translator
         if TRANSLATORS[backend].supported_features['change-instance']:
-            self.backend_instance_row.set_visible(True)
-            self.backend_instance_label.set_label(Settings.get().instance_url)
+            self.instance_entry.set_visible(True)
+            self.instance_entry.set_text(Settings.get().instance_url)
         else:
-            self.backend_instance_row.set_visible(False)
+            self.instance_entry.set_visible(False)
 
         if TRANSLATORS[backend].supported_features['api-key-supported']:
-            self.api_key_row.set_visible(True)
-            self.api_key_label.set_label(Settings.get().api_key or 'None')
+            self.api_key_entry.set_visible(True)
+            self.api_key_entry.set_text(Settings.get().api_key)
         else:
-            self.api_key_row.set_visible(False)
+            self.api_key_entry.set_visible(False)
 
 
 class BackendObject(GObject.Object):
