@@ -30,9 +30,9 @@ class DialectWindow(Adw.ApplicationWindow):
     main_stack = Gtk.Template.Child()
     error_page = Gtk.Template.Child()
     translator_box = Gtk.Template.Child()
-    retry_backend_btn = Gtk.Template.Child()
     key_page = Gtk.Template.Child()
     rmv_key_btn = Gtk.Template.Child()
+    error_api_key_btn = Gtk.Template.Child()
 
     title_stack = Gtk.Template.Child()
     langs_button_box = Gtk.Template.Child()
@@ -50,7 +50,6 @@ class DialectWindow(Adw.ApplicationWindow):
     mistakes = Gtk.Template.Child()
     mistakes_label = Gtk.Template.Child()
     char_counter = Gtk.Template.Child()
-    src_scroller = Gtk.Template.Child()
     src_text = Gtk.Template.Child()
     clear_btn = Gtk.Template.Child()
     paste_btn = Gtk.Template.Child()
@@ -60,7 +59,6 @@ class DialectWindow(Adw.ApplicationWindow):
     dest_box = Gtk.Template.Child()
     dest_pron_revealer = Gtk.Template.Child()
     dest_pron_label = Gtk.Template.Child()
-    dest_scroller = Gtk.Template.Child()
     dest_text = Gtk.Template.Child()
     dest_toolbar_stack = Gtk.Template.Child()
     trans_spinner = Gtk.Template.Child()
@@ -149,8 +147,6 @@ class DialectWindow(Adw.ApplicationWindow):
         self.set_help_overlay(DialectShortcutsWindow())
 
         # Load translator
-        self.retry_backend_btn.connect('clicked', self.retry_load_translator)
-        self.rmv_key_btn.connect('clicked', self.remove_key_and_reload)
         self.load_translator(True)
         # Get languages available for speech
         if Settings.get().active_tts != '':
@@ -309,6 +305,7 @@ class DialectWindow(Adw.ApplicationWindow):
                         _('Please set a valid API key or unset the API key in the preferences.')
                     )
                     self.rmv_key_btn.set_visible(True)
+                    self.error_api_key_btn.set_visible(True)
                 self.main_stack.set_visible_child_name('api-key')
             except (TranslatorError, ResponseEmpty) as exc:
                 logging.warning(exc)
@@ -360,9 +357,11 @@ class DialectWindow(Adw.ApplicationWindow):
         self.error_page.set_title(title)
         self.error_page.set_description(description)
 
+    @Gtk.Template.Callback()
     def retry_load_translator(self, _button):
         self.load_translator()
 
+    @Gtk.Template.Callback()
     def remove_key_and_reload(self, _button):
         Settings.get().reset_api_key()
         self.load_translator()
@@ -469,15 +468,11 @@ class DialectWindow(Adw.ApplicationWindow):
         self.win_key_ctrlr.connect('key-pressed', self.on_key_event)
         # "Did you mean" links
         self.mistakes_label.connect('activate-link', self.on_mistakes_clicked)
-        self.src_scroller.get_vadjustment().connect('value-changed', self.on_src_scrolled)
-        self.src_scroller.get_vadjustment().connect('changed', self.on_src_scrolled)
 
         # Right buffer
         self.dest_buffer = self.dest_text.get_buffer()
         self.dest_buffer.set_text('')
         self.dest_buffer.connect('changed', self.on_dest_text_changed)
-        self.dest_scroller.get_vadjustment().connect('value-changed', self.on_dest_scrolled)
-        self.dest_scroller.get_vadjustment().connect('changed', self.on_dest_scrolled)
         # Translation progress spinner
         self.trans_spinner.hide()
         self.trans_warning.hide()
@@ -605,20 +600,6 @@ class DialectWindow(Adw.ApplicationWindow):
             self.dest_voice_btn.set_child(self.dest_voice_image)
             self.dest_voice_spinner.stop()
 
-    def on_src_scrolled(self, vadj):
-        if (vadj.get_value() + vadj.get_page_size() != vadj.get_upper()
-                or (self.src_pron_revealer.get_reveal_child() or self.mistakes.get_reveal_child())):
-            self.src_scroller.get_style_context().add_class('scroller-border')
-        else:
-            self.src_scroller.get_style_context().remove_class('scroller-border')
-
-    def on_dest_scrolled(self, vadj):
-        if (vadj.get_value() + vadj.get_page_size() != vadj.get_upper()
-                or self.dest_pron_revealer.get_reveal_child()):
-            self.dest_scroller.get_style_context().add_class('scroller-border')
-        else:
-            self.dest_scroller.get_style_context().remove_class('scroller-border')
-
     def on_src_lang_changed(self, _obj, _param):
         code = self.src_lang_selector.get_property('selected')
         dest_code = self.dest_lang_selector.get_property('selected')
@@ -638,6 +619,11 @@ class DialectWindow(Adw.ApplicationWindow):
                 code in self.tts_langs and src_text != ''
             )
 
+        # Disable or enable switch function.
+        self.lookup_action('switch').set_enabled(
+            code != 'auto'
+        )
+
         if code in self.translator.languages:
             self.src_lang_label.set_label(get_lang_name(code))
             # Update saved src langs list
@@ -660,7 +646,8 @@ class DialectWindow(Adw.ApplicationWindow):
         if self.translator.supported_features['detection']:
             self.src_lang_selector.insert_recent('auto', _('Auto'))
         for code in self.src_langs:
-            self.src_lang_selector.insert_recent(code, get_lang_name(code))
+            if code in self.translator.languages:
+                self.src_lang_selector.insert_recent(code, get_lang_name(code))
 
         # Refresh list
         self.src_lang_selector.refresh_selected()
@@ -698,7 +685,8 @@ class DialectWindow(Adw.ApplicationWindow):
         # Rewrite recent langs
         self.dest_lang_selector.clear_recent()
         for code in self.dest_langs:
-            self.dest_lang_selector.insert_recent(code, get_lang_name(code))
+            if code in self.translator.languages:
+                self.dest_lang_selector.insert_recent(code, get_lang_name(code))
 
         # Refresh list
         self.dest_lang_selector.refresh_selected()
@@ -744,14 +732,6 @@ class DialectWindow(Adw.ApplicationWindow):
         self.langs_button_box.set_sensitive(True)
         self.lookup_action('translation').set_enabled(self.src_buffer.get_char_count() != 0)
 
-    def switch_auto_lang(self, dest_language, src_text, dest_text):
-        src_language = self.translator.detect(src_text).lang
-        if isinstance(src_language, list):
-            src_language = src_language[0]
-
-        # Switch all
-        GLib.idle_add(self.switch_all, src_language, dest_language, src_text, dest_text)
-
     def ui_switch(self, _action, _param):
         # Get variables
         self.langs_button_box.set_sensitive(False)
@@ -769,15 +749,7 @@ class DialectWindow(Adw.ApplicationWindow):
             True
         )
         if src_language == 'auto':
-            if src_text == '':
-                src_language = self.src_langs[0]
-            else:
-                threading.Thread(
-                    target=self.switch_auto_lang,
-                    args=(dest_language, src_text, dest_text),
-                    daemon=True
-                ).start()
-                return
+            return
 
         # Switch all
         self.switch_all(src_language, dest_language, src_text, dest_text)
@@ -1158,7 +1130,7 @@ class DialectWindow(Adw.ApplicationWindow):
         # Pronunciation
         reveal = Settings.get().show_pronunciation
         if self.translator.supported_features['pronunciation']:
-            if self.trans_src_pron is not None:
+            if self.trans_src_pron is not None and self.trans_mistakes == [None, None]:
                 self.src_pron_label.set_text(self.trans_src_pron)
                 self.src_pron_revealer.set_reveal_child(reveal)
             elif self.src_pron_revealer.get_reveal_child():
@@ -1235,8 +1207,5 @@ class DialectWindow(Adw.ApplicationWindow):
     def reload_backends(self):
         self.set_property('backend-loading', True)
 
-        # Save previous backend settings
-        self.save_settings()
-
         # Load translator
         self.load_translator()
